@@ -2,97 +2,139 @@ package Math::Random::Secure::RNG;
 BEGIN {
   $Math::Random::Secure::RNG::VERSION = '0.06';
 }
-use Any::Moose qw(Moose ::Util::TypeConstraints);
+use Moo;
 use Math::Random::ISAAC;
 use Crypt::Random::Source::Factory;
 use constant ON_WINDOWS => $^O =~ /Win32/i ? 1 : 0;
 use if ON_WINDOWS, 'Crypt::Random::Source::Strong::Win32';
 
-subtype 'MRS::RNG::Seed' => as 'Str' => where { _check_seed($_) };
-subtype 'MRS::RNG::Rng' => as 'Object' 
-    => where { $_->can('irand') && $_->can('rand') };
-
-has seeder => (is => 'ro', isa => 'Crypt::Random::Source::Base', 
-               lazy_build => 1);
+has seeder => (
+  is => 'ro',
+  lazy => 1,
+  builder => '_build_seeder',
+);
 # Default to a 512-bit key, which should be impossible to break. I wrote
 # to the author of ISAAC and he said it's fine to not use a full 256
 # integers to seed ISAAC.
-has seed_size => (is => 'ro', isa => 'Int', default => 64);
-has seed => (is => 'rw', isa => 'MRS::RNG::Seed', lazy_build => 1,
-             clearer => 'clear_seed', predicate => 'has_seed');
-has rng => (is => 'ro', isa => 'MRS::RNG::Rng', lazy_build => 1, 
-            handles => ['irand', 'rand'], clearer => 'clear_rng');
+has seed_size => (
+  is => 'ro',
+  # isa => 'Int',
+  default => 64,
+);
+
+has seed => (
+  is => 'rw',
+  isa => \&_check_seed,
+  lazy => 1,
+  builder => '_build_seed',
+  clearer => 'clear_seed',
+  predicate => 'has_seed',
+);
+
+has rng => (
+  is => 'ro',
+  lazy => 1,
+  builder => '_build_rng',
+  handles => ['irand', 'rand'],
+  clearer => 'clear_rng',
+);
+
+has _for_pid => (
+  is => 'rw',
+  default => sub { $$ },
+);
+
+sub _clear_for_pid { shift->_for_pid($$) }
+
+before qw(irand rand) => '_maybe_clear_seed';
+
+sub _maybe_clear_seed {
+  my $self = shift;
+
+  if ($self->_for_pid != $$) {
+    $self->clear_seed;
+    $self->_clear_for_pid;
+  }
+}
+
 
 sub BUILD {
-    my ($self) = @_;
+  my ($self) = @_;
 
-    if ($self->seed_size < 8) {
-        warn "Setting seed_size to less than 8 is not recommended";
-    }
+  if ($self->seed_size < 8) {
+    warn "Setting seed_size to less than 8 is not recommended";
+  }
 }
 
 sub _check_seed {
-    my ($seed) = @_;
-    if (length($seed) < 8) {
-        warn "Your seed is less than 8 bytes (64 bits). It could be"
-             . " easy to crack";
-    }
-    # If it looks like we were seeded with a 32-bit integer, warn the
-    # user that they are making a dangerous, easily-crackable mistake.
-    # We do this during BUILD so that it happens during srand() in
-    # Math::Secure::RNG.
-    elsif (length($seed) <= 10 and $seed =~ /^\d+$/) {
-        warn "RNG seeded with a 32-bit integer, this is easy to crack";
-    }
+  my ($seed) = @_;
+  if (length($seed) < 8) {
+    warn "Your seed is less than 8 bytes (64 bits). It could be"
+      . " easy to crack";
+  }
+  # If it looks like we were seeded with a 32-bit integer, warn the
+  # user that they are making a dangerous, easily-crackable mistake.
+  # We do this during BUILD so that it happens during srand() in
+  # Math::Secure::RNG.
+  elsif (length($seed) <= 10 and $seed =~ /^\d+$/) {
+    warn "RNG seeded with a 32-bit integer, this is easy to crack";
+  }
 
-    # _check_seed is used as a type constraint, so needs to return 1.
-    return 1;
+  # _check_seed is used as a type constraint, so needs to return 1.
+  return 1;
 }
 
 sub _build_seeder {
-    my $factory = Crypt::Random::Source::Factory->new();
-    # On Windows, we want to always pick Crypt::Random::Source::Strong::Win32,
-    # which this will do.
-    if (ON_WINDOWS) {
-        return $factory->get_strong;
-    }
+  my $factory = Crypt::Random::Source::Factory->new();
+  # On Windows, we want to always pick Crypt::Random::Source::Strong::Win32,
+  # which this will do.
+  if (ON_WINDOWS) {
+    return $factory->get_strong;
+  }
 
-    my $source = $factory->get;
-    # Never allow rand() to be used as a source, it cannot possibly be
-    # cryptographically strong with 2^15 or 2^32 bits for its seed.
-    if ($source->isa('Crypt::Random::Source::Weak::rand')) {
-        $source = $factory->get_strong;
-    }
-    return $source;
+  my $source = $factory->get;
+  # Never allow rand() to be used as a source, it cannot possibly be
+  # cryptographically strong with 2^15 or 2^32 bits for its seed.
+  if ($source->isa('Crypt::Random::Source::Weak::rand')) {
+    $source = $factory->get_strong;
+  }
+  return $source;
 }
 
 sub _build_seed {
-    my ($self) = @_;
-    return $self->seeder->get($self->seed_size);
+  my ($self) = @_;
+  return $self->seeder->get($self->seed_size);
 }
 
 sub _build_rng {
-    my ($self) = @_;
-    my @seed_ints = unpack('L*', $self->seed);
-    my $rng = Math::Random::ISAAC->new(@seed_ints);
-    # One part of having a cryptographically-secure RNG is not being
-    # able to see the seed in the internal state of the RNG.
-    $self->clear_seed;
-    # It's faster to skip the frontend interface of Math::Random::ISAAC
-    # and just use the backend directly. However, in case the internal
-    # code of Math::Random::ISAAC changes at some point, we do make sure
-    # that the {backend} element actually exists first.
-    return $rng->{backend} ? $rng->{backend} : $rng;
+  my ($self) = @_;
+  my @seed_ints = unpack('L*', $self->seed);
+  my $rng = Math::Random::ISAAC->new(@seed_ints);
+  # One part of having a cryptographically-secure RNG is not being
+  # able to see the seed in the internal state of the RNG.
+  $self->clear_seed;
+  # It's faster to skip the frontend interface of Math::Random::ISAAC
+  # and just use the backend directly. However, in case the internal
+  # code of Math::Random::ISAAC changes at some point, we do make sure
+  # that the {backend} element actually exists first.
+  return $rng->{backend} ? $rng->{backend} : $rng;
 }
 
-__PACKAGE__->meta->make_immutable;
-__PACKAGE__;
+1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-Math::Random::Secure::RNG - The underlying PRNG, as an object.
+Math::Random::Secure::RNG
+
+=head1 VERSION
+
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -112,6 +154,10 @@ to a random-number generator, you can use this.
 Math::Random::Secure::RNG uses L<Any::Moose>, meaning that it has a
 C<new> method that works like L<Mouse> or L<Moose> modules work.
 
+=head1 NAME
+
+Math::Random::Secure::RNG - The underlying PRNG, as an object.
+
 =head1 METHODS
 
 =head2 irand
@@ -137,8 +183,8 @@ L<Math::Random::ISAAC>.
 
 The random data used to seed L</rng>, as a string of bytes. This should
 be large enough to properly seed L</rng>. This means I<minimally>, it
-should be 8 bytes (64 bits) and more ideally, 32 bytes (256 bits) or 64 
-bytes (512 bits). For an idea of how large your seed should be, see 
+should be 8 bytes (64 bits) and more ideally, 32 bytes (256 bits) or 64
+bytes (512 bits). For an idea of how large your seed should be, see
 L<http://burtleburtle.net/bob/crypto/magnitude.html#brute> for information
 on how long it would take to brute-force seeds of each size.
 
@@ -149,7 +195,7 @@ be very random. B<There are serious attacks possible against random number
 generators that are seeded with non-random data or with insufficient random
 data.>
 
-By default, we use a 512-bit (64 byte) seed. If 
+By default, we use a 512-bit (64 byte) seed. If
 L<Moore's Law|http://en.wikipedia.org/wiki/Moore's_law> continues to hold
 true, it will be approximately 1000 years before computers can brute-force a
 512-bit (64 byte) seed at any reasonable speed (and physics suggests that
@@ -174,3 +220,27 @@ See L</seed> for more info about what is a reasonable seed size.
 =head1 SEE ALSO
 
 L<Math::Random::Secure>
+
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Max Kanat-Alexander <mkanat@cpan.org>
+
+=item *
+
+Arthur Axel "fREW" Schmidt <math-random-secure@afoolishmanifesto.com>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2010 by BugzillaSource, Inc.
+
+This is free software, licensed under:
+
+  The Artistic License 2.0 (GPL Compatible)
+
+=cut
